/**
 * @class TheTmp
 */
'use strict'

const tmp = require('tmp')

/** @lends TheTmp */
class TheTmp {
  /**
   * Generate tmp file
   * @param {Object} [options={}] - Optional settings
   * @returns {Promise<Object>}
   */
  async generate (options = {}) {
    return await new Promise((resolve, reject) => {
      tmp.file(options, (err, filename, descriptor, cleanup) => {
        if (err) {
          reject(err)
          return
        }
        resolve({ cleanup, descriptor, filename })
      })
    })
  }

  /**
   * Create and use tmp file until handler resolved
   * @param {function} handler - Handler function to use tmp file
   * @param {Object} options - Tmp options
   * @returns {Promise<*>}
   */
  async while (handler, options = {}) {
    const { cleanup, descriptor, filename } = await this.generate(options)
    try {
      await handler(filename, descriptor)
    } finally {
      cleanup()
    }
  }
}

module.exports = TheTmp
